/* 
   v4l1_radio_core.c (C) Antonio Beamud 2004 (antonio.beamud@linkend.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <errno.h>
#include <unistd.h>

#include <linux/videodev.h>
#include "v4l1_core.c"

static int get_freq_fact(int fd) 
{
	struct video_tuner tuner;
	
	tuner.tuner = 0;
	if (ioctl (fd, VIDIOCGTUNER, &tuner) < 0)
		return 16;
	if ((tuner.flags & VIDEO_TUNER_LOW) == 0)
		return 16;
	return 16000;
}

void v4l1_radio_core_close(struct cap_core_t * cct){
	
	struct video_audio vid_aud;
	struct v4l1_core_t * vct = (struct v4l1_core_t *)cct->priv;

	if (ioctl(vct->fd, VIDIOCGAUDIO, &vid_aud))
		perror("VIDIOCGAUDIO");
	vid_aud.flags |= VIDEO_AUDIO_MUTE;

	DXPRINT("enter\n");
	if(!vct)
		return;
	DXPRINT("setup mute\n");
	if (ioctl(vct->fd, VIDIOCSAUDIO, &vid_aud))
		perror("VIDIOCSAUDIO");
	DXPRINT("close fd\n");
	close(vct->fd);
	DXPRINT("free vct\n");
	free(vct);
	DXPRINT("free cct\n");
	free(cct);
	DXPRINT("leave\n");
	return;
}

int v4l1_radio_core_signal_present(struct cap_core_t * cct)
{
	struct v4l1_core_t * vct = (struct v4l1_core_t *)cct->priv;
	int signal;
	struct video_tuner tuner;
	
	memset(&tuner,0,sizeof(tuner));
	ioctl(vct->fd, VIDIOCGTUNER, &tuner);
	signal = tuner.signal>>14;
	return signal;
}

struct cap_core_t * v4l1_radio_core_init(char ** args, unsigned int **pfmt_p)
{
	struct cap_core_t * cct;
	struct v4l1_core_t * vct;
	struct video_audio vid_aud;

	char * fdev = NULL;
	double freq = -1;
	NV_ARGS;
	
	DPRINT("v4l1_radio_core_init - Starting...\n");

	cct = malloc(sizeof(struct cap_core_t));
	if(!cct) {
		PERROR("Could no malloc new cap core");
		return NULL;
	}
	vct = malloc(sizeof(struct v4l1_core_t));
	if(!vct) {
		PERROR("Could no malloc new v4l1 core");
		free(cct);
		return NULL;
	}
	memset(vct, 0, sizeof(struct v4l1_core_t));
	memcpy(cct, &v4l1_radio_core, sizeof(struct cap_core_t));
	cct->priv = (void *)vct;
	
	NV_SCAN_ARGS {
		if(strcmp(NV_THIS_ARG, "-v") == 0) {
			if(!NV_NEXT_ARG) {
				EPRINT("Missing argument to -v.\n");
				goto usage;
			}
			fdev = NV_THIS_ARG;
		
		} else if(strcmp(NV_THIS_ARG, "-frequency") == 0) {
			if(!NV_NEXT_ARG) { 
				EPRINT("Missing argument to -frequency.\n");
				goto usage;
			}
			freq = atof(NV_THIS_ARG);
			if(freq < 0) {
				EPRINT("Invalid argument to -frequency (>=0).\n");
				goto usage;
			}
		}
	}
	
	vct->fd = open(fdev, O_RDONLY);
	if(vct->fd<=0) {
		PERROR("Open capture device");
		goto error0;
	}
	/* Should be radio because of dynamic link of this object */
	/* if(strcmp(imp, "radio"))*/

	/* Try to mute device */
	
	DPRINT("Trying to mute radio device\n");
	
	if (ioctl(vct->fd, VIDIOCGAUDIO, &vid_aud))
		PERROR("VIDIOCGAUDIO");
	vid_aud.flags |= VIDEO_AUDIO_MUTE;
	
	if (ioctl(vct->fd, VIDIOCSAUDIO, &vid_aud))
		PERROR("VIDIOCSAUDIO");
	
	/* vct->freq = freq; */
	
	if(freq > 0) {
		int ifreq = (freq+1.0/32)*get_freq_fact(vct->fd);
		if(ioctl(vct->fd, VIDIOCSFREQ, &ifreq)) {
			PERROR("Could not set frequency");
			goto error1;
		}
	}
	
	/* Unmute */
	if (ioctl(vct->fd, VIDIOCGAUDIO, &vid_aud))
		perror("VIDIOCGAUDIO");
	if (vid_aud.volume == 0)
		vid_aud.volume = 65535;
	vid_aud.flags &= ~VIDEO_AUDIO_MUTE;
	if (ioctl(vct->fd, VIDIOCSAUDIO, &vid_aud))
		perror("VIDIOCSAUDIO");
	
	IPRINT("Radio frequency: %.1f\n", freq);
	IPRINT("Radio device: %s\n", fdev);
	
	return cct;
error1:
	close(vct->fd);
error0:
	free(cct);
	free(vct);
	return NULL;
usage:
	EPRINT("Error parsing options.  Please run without any command line options for\n");
	EPRINT("detailed usage information.\n");
	free(cct);
	free(vct);
	return NULL;
}

inline ntime_t v4l1_radio_core_ftime(struct cap_core_t * cct)
{
	DPRINT("Setting ftime to 40000000ULL\n");
	return 40000000ULL;
}

void v4l1_radio_core_usage()
{
	MPRINT("v4l1 radio capture core usage:\n");
	MPRINT("\t-v <device name> capture device name  [/dev/radio]\n");
	MPRINT("\t-frequency <MHz> set input frequency  [unchanged]\n");
}


struct cap_core_t v4l1_radio_core = {
	cap_core_init: v4l1_radio_core_init,
	cap_core_close: v4l1_radio_core_close,
	cap_core_next: NULL,
	cap_core_status: NULL,
	cap_core_ftime: v4l1_radio_core_ftime,
	cap_core_overrides: NULL,
	cap_core_usage: v4l1_radio_core_usage,
	cap_core_signal_present: v4l1_radio_core_signal_present,
	priv: NULL
};

